#define VALA_FREE_CHECKED(o,f) ((o) == NULL ? NULL : ((o) = (f (o), NULL)))

#include <src/ChannelList.h>
#include <gee/map.h>
#include "src/ChannelListReader.h"




struct _DVBChannelListPrivate {
	GFile* _channels_file;
	GStaticRecMutex __lock_channels;
	GFile* channelsfile;
};

#define DVB_CHANNEL_LIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), DVB_TYPE_CHANNEL_LIST, DVBChannelListPrivate))
enum  {
	DVB_CHANNEL_LIST_DUMMY_PROPERTY,
	DVB_CHANNEL_LIST_CHANNELS_FILE,
	DVB_CHANNEL_LIST_SIZE
};
static GType dvb_channel_list_real_get_element_type (GeeIterable* base);
static GeeIterator* dvb_channel_list_real_iterator (GeeIterable* base);
static guint* dvb_channel_list_real_GetChannels (DVBIDBusChannelList* base, int* result_length1);
static guint* dvb_channel_list_real_GetRadioChannels (DVBIDBusChannelList* base, int* result_length1);
static guint* dvb_channel_list_real_GetTVChannels (DVBIDBusChannelList* base, int* result_length1);
static char* dvb_channel_list_real_GetChannelName (DVBIDBusChannelList* base, guint channel_id);
static char* dvb_channel_list_real_GetChannelNetwork (DVBIDBusChannelList* base, guint channel_id);
static gboolean dvb_channel_list_real_IsRadioChannel (DVBIDBusChannelList* base, guint channel_id);
static void dvb_channel_list_set_channels_file (DVBChannelList* self, GFile* value);
static GObject * dvb_channel_list_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static gpointer dvb_channel_list_parent_class = NULL;
static GeeIterableIface* dvb_channel_list_gee_iterable_parent_iface = NULL;
static DVBIDBusChannelListIface* dvb_channel_list_dvb_id_bus_channel_list_parent_iface = NULL;
static void dvb_channel_list_finalize (GObject* obj);



DVBChannelList* dvb_channel_list_construct (GType object_type, GFile* channelsfile) {
	GParameter * __params;
	GParameter * __params_it;
	DVBChannelList * self;
	g_return_val_if_fail (channelsfile == NULL || G_IS_FILE (channelsfile), NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "channels-file";
	g_value_init (&__params_it->value, G_TYPE_FILE);
	g_value_set_object (&__params_it->value, channelsfile);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


DVBChannelList* dvb_channel_list_new (GFile* channelsfile) {
	return dvb_channel_list_construct (DVB_TYPE_CHANNEL_LIST, channelsfile);
}


DVBChannel* dvb_channel_list_get (DVBChannelList* self, guint sid) {
	DVBChannel* val;
	g_return_val_if_fail (DVB_IS_CHANNEL_LIST (self), NULL);
	val = NULL;
	g_static_rec_mutex_lock (&self->priv->__lock_channels);
	{
		if (gee_map_contains (GEE_MAP (self->channels), GUINT_TO_POINTER (sid))) {
			DVBChannel* _tmp0;
			_tmp0 = NULL;
			val = (_tmp0 = ((DVBChannel*) (gee_map_get (GEE_MAP (self->channels), GUINT_TO_POINTER (sid)))), (val == NULL ? NULL : (val = (g_object_unref (val), NULL))), _tmp0);
		}
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_channels);
	return val;
}


void dvb_channel_list_add (DVBChannelList* self, DVBChannel* channel) {
	g_return_if_fail (DVB_IS_CHANNEL_LIST (self));
	g_return_if_fail (DVB_IS_CHANNEL (channel));
	g_static_rec_mutex_lock (&self->priv->__lock_channels);
	{
		gee_map_set (GEE_MAP (self->channels), GUINT_TO_POINTER (dvb_channel_get_Sid (channel)), channel);
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_channels);
}


void dvb_channel_list_remove (DVBChannelList* self, guint sid) {
	g_return_if_fail (DVB_IS_CHANNEL_LIST (self));
	g_static_rec_mutex_lock (&self->priv->__lock_channels);
	{
		gee_map_remove (GEE_MAP (self->channels), GUINT_TO_POINTER (sid));
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_channels);
}


gboolean dvb_channel_list_contains (DVBChannelList* self, guint sid) {
	gboolean val;
	g_return_val_if_fail (DVB_IS_CHANNEL_LIST (self), FALSE);
	val = FALSE;
	g_static_rec_mutex_lock (&self->priv->__lock_channels);
	{
		val = gee_map_contains (GEE_MAP (self->channels), GUINT_TO_POINTER (sid));
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_channels);
	return val;
}


void dvb_channel_list_clear (DVBChannelList* self) {
	g_return_if_fail (DVB_IS_CHANNEL_LIST (self));
	g_static_rec_mutex_lock (&self->priv->__lock_channels);
	{
		gee_map_clear (GEE_MAP (self->channels));
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_channels);
}


static GType dvb_channel_list_real_get_element_type (GeeIterable* base) {
	DVBChannelList * self;
	self = DVB_CHANNEL_LIST (base);
	return DVB_TYPE_CHANNEL;
}


static GeeIterator* dvb_channel_list_real_iterator (GeeIterable* base) {
	DVBChannelList * self;
	GeeCollection* _tmp0;
	GeeIterator* _tmp1;
	self = DVB_CHANNEL_LIST (base);
	_tmp0 = NULL;
	_tmp1 = NULL;
	return (_tmp1 = gee_iterable_iterator (GEE_ITERABLE ((_tmp0 = gee_map_get_values (GEE_MAP (self->channels))))), (_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL))), _tmp1);
}


DVBChannelList* dvb_channel_list_restore_from_file (GFile* channelsfile, DVBAdapterType type, GError** error) {
	GError * inner_error;
	DVBChannelListReader* reader;
	DVBChannelList* _tmp0;
	g_return_val_if_fail (G_IS_FILE (channelsfile), NULL);
	inner_error = NULL;
	/* FIXME make thread-safe*/
	reader = dvb_channel_list_reader_new (channelsfile, type);
	_tmp0 = NULL;
	return (_tmp0 = dvb_channel_list_reader_read (reader, &inner_error), (reader == NULL ? NULL : (reader = (g_object_unref (reader), NULL))), _tmp0);
}


/**
         * @returns: List of channel IDs aka SIDs
         */
static guint* dvb_channel_list_real_GetChannels (DVBIDBusChannelList* base, int* result_length1) {
	DVBChannelList * self;
	guint* _tmp1;
	gint ids_length1;
	gint _tmp0;
	guint* ids;
	gint i;
	guint* _tmp2;
	self = DVB_CHANNEL_LIST (base);
	_tmp1 = NULL;
	ids = (_tmp1 = g_new0 (guint, (_tmp0 = dvb_channel_list_get_size (self))), ids_length1 = _tmp0, _tmp1);
	i = 0;
	g_static_rec_mutex_lock (&self->priv->__lock_channels);
	{
		{
			GeeSet* id_collection;
			GeeIterator* id_it;
			id_collection = gee_map_get_keys (GEE_MAP (self->channels));
			id_it = gee_iterable_iterator (GEE_ITERABLE (id_collection));
			while (gee_iterator_next (id_it)) {
				guint id;
				id = GPOINTER_TO_UINT (gee_iterator_get (id_it));
				{
					ids[i] = id;
					i++;
				}
			}
			(id_it == NULL ? NULL : (id_it = (g_object_unref (id_it), NULL)));
			(id_collection == NULL ? NULL : (id_collection = (g_object_unref (id_collection), NULL)));
		}
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_channels);
	_tmp2 = NULL;
	return (_tmp2 = ids, (*result_length1) = ids_length1, _tmp2);
}


/**
         * @returns: List of channel IDs aka SIDs of radio channels
         */
static guint* dvb_channel_list_real_GetRadioChannels (DVBIDBusChannelList* base, int* result_length1) {
	DVBChannelList * self;
	GSList* radio_channels;
	guint* _tmp1;
	gint ids_length1;
	gint _tmp0;
	guint* ids;
	guint* _tmp2;
	guint* _tmp3;
	self = DVB_CHANNEL_LIST (base);
	radio_channels = NULL;
	g_static_rec_mutex_lock (&self->priv->__lock_channels);
	{
		{
			GeeSet* id_collection;
			GeeIterator* id_it;
			id_collection = gee_map_get_keys (GEE_MAP (self->channels));
			id_it = gee_iterable_iterator (GEE_ITERABLE (id_collection));
			while (gee_iterator_next (id_it)) {
				guint id;
				id = GPOINTER_TO_UINT (gee_iterator_get (id_it));
				{
					DVBChannel* chan;
					chan = ((DVBChannel*) (gee_map_get (GEE_MAP (self->channels), GUINT_TO_POINTER (id))));
					if (dvb_channel_get_VideoPID (chan) == 0) {
						radio_channels = g_slist_prepend (radio_channels, GUINT_TO_POINTER (id));
					}
					(chan == NULL ? NULL : (chan = (g_object_unref (chan), NULL)));
				}
			}
			(id_it == NULL ? NULL : (id_it = (g_object_unref (id_it), NULL)));
			(id_collection == NULL ? NULL : (id_collection = (g_object_unref (id_collection), NULL)));
		}
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_channels);
	radio_channels = g_slist_reverse (radio_channels);
	_tmp1 = NULL;
	ids = (_tmp1 = g_new0 (guint, (_tmp0 = g_slist_length (radio_channels))), ids_length1 = _tmp0, _tmp1);
	{
		gint i;
		i = 0;
		for (; i < g_slist_length (radio_channels); i++) {
			ids[i] = GPOINTER_TO_UINT (GPOINTER_TO_UINT (g_slist_nth_data (radio_channels, ((guint) (i)))));
		}
	}
	_tmp2 = NULL;
	_tmp3 = NULL;
	return (_tmp3 = (_tmp2 = ids, (*result_length1) = ids_length1, _tmp2), (radio_channels == NULL ? NULL : (radio_channels = (g_slist_free (radio_channels), NULL))), _tmp3);
}


/**
         * @returns: List of channel IDs aka SIDs of TV channels
         */
static guint* dvb_channel_list_real_GetTVChannels (DVBIDBusChannelList* base, int* result_length1) {
	DVBChannelList * self;
	GSList* video_channels;
	guint* _tmp1;
	gint ids_length1;
	gint _tmp0;
	guint* ids;
	guint* _tmp2;
	guint* _tmp3;
	self = DVB_CHANNEL_LIST (base);
	video_channels = NULL;
	g_static_rec_mutex_lock (&self->priv->__lock_channels);
	{
		{
			GeeSet* id_collection;
			GeeIterator* id_it;
			id_collection = gee_map_get_keys (GEE_MAP (self->channels));
			id_it = gee_iterable_iterator (GEE_ITERABLE (id_collection));
			while (gee_iterator_next (id_it)) {
				guint id;
				id = GPOINTER_TO_UINT (gee_iterator_get (id_it));
				{
					DVBChannel* chan;
					chan = ((DVBChannel*) (gee_map_get (GEE_MAP (self->channels), GUINT_TO_POINTER (id))));
					if (dvb_channel_get_VideoPID (chan) != 0) {
						video_channels = g_slist_prepend (video_channels, GUINT_TO_POINTER (id));
					}
					(chan == NULL ? NULL : (chan = (g_object_unref (chan), NULL)));
				}
			}
			(id_it == NULL ? NULL : (id_it = (g_object_unref (id_it), NULL)));
			(id_collection == NULL ? NULL : (id_collection = (g_object_unref (id_collection), NULL)));
		}
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_channels);
	video_channels = g_slist_reverse (video_channels);
	_tmp1 = NULL;
	ids = (_tmp1 = g_new0 (guint, (_tmp0 = g_slist_length (video_channels))), ids_length1 = _tmp0, _tmp1);
	{
		gint i;
		i = 0;
		for (; i < g_slist_length (video_channels); i++) {
			ids[i] = GPOINTER_TO_UINT (GPOINTER_TO_UINT (g_slist_nth_data (video_channels, ((guint) (i)))));
		}
	}
	_tmp2 = NULL;
	_tmp3 = NULL;
	return (_tmp3 = (_tmp2 = ids, (*result_length1) = ids_length1, _tmp2), (video_channels == NULL ? NULL : (video_channels = (g_slist_free (video_channels), NULL))), _tmp3);
}


/**
         * @channel_id: ID of channel
         * @returns: Name of channel if channel with id exists
         * otherwise an empty string
         */
static char* dvb_channel_list_real_GetChannelName (DVBIDBusChannelList* base, guint channel_id) {
	DVBChannelList * self;
	char* val;
	self = DVB_CHANNEL_LIST (base);
	val = g_strdup ("");
	g_static_rec_mutex_lock (&self->priv->__lock_channels);
	{
		if (gee_map_contains (GEE_MAP (self->channels), GUINT_TO_POINTER (channel_id))) {
			char* _tmp2;
			const char* _tmp1;
			DVBChannel* _tmp0;
			_tmp2 = NULL;
			_tmp1 = NULL;
			_tmp0 = NULL;
			val = (_tmp2 = (_tmp1 = dvb_channel_get_Name ((_tmp0 = ((DVBChannel*) (gee_map_get (GEE_MAP (self->channels), GUINT_TO_POINTER (channel_id)))))), (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (val = (g_free (val), NULL)), _tmp2);
			(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
		}
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_channels);
	return val;
}


/**
         * @channel_id: ID of channel
         * @returns: Name of network the channel belongs to
         * if the channel with id exists, otherwise an empty
         * string
         */
static char* dvb_channel_list_real_GetChannelNetwork (DVBIDBusChannelList* base, guint channel_id) {
	DVBChannelList * self;
	char* val;
	self = DVB_CHANNEL_LIST (base);
	val = g_strdup ("");
	g_static_rec_mutex_lock (&self->priv->__lock_channels);
	{
		if (gee_map_contains (GEE_MAP (self->channels), GUINT_TO_POINTER (channel_id))) {
			char* _tmp2;
			const char* _tmp1;
			DVBChannel* _tmp0;
			_tmp2 = NULL;
			_tmp1 = NULL;
			_tmp0 = NULL;
			val = (_tmp2 = (_tmp1 = dvb_channel_get_Network ((_tmp0 = ((DVBChannel*) (gee_map_get (GEE_MAP (self->channels), GUINT_TO_POINTER (channel_id)))))), (_tmp1 == NULL ? NULL : g_strdup (_tmp1))), (val = (g_free (val), NULL)), _tmp2);
			(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
		}
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_channels);
	return val;
}


/**
         * @channel_id: ID of channel
         * @returns: Whether the channel is a radio channel or not
         */
static gboolean dvb_channel_list_real_IsRadioChannel (DVBIDBusChannelList* base, guint channel_id) {
	DVBChannelList * self;
	gboolean val;
	self = DVB_CHANNEL_LIST (base);
	val = FALSE;
	g_static_rec_mutex_lock (&self->priv->__lock_channels);
	{
		if (gee_map_contains (GEE_MAP (self->channels), GUINT_TO_POINTER (channel_id))) {
			DVBChannel* _tmp0;
			_tmp0 = NULL;
			val = (dvb_channel_get_VideoPID ((_tmp0 = ((DVBChannel*) (gee_map_get (GEE_MAP (self->channels), GUINT_TO_POINTER (channel_id)))))) == 0);
			(_tmp0 == NULL ? NULL : (_tmp0 = (g_object_unref (_tmp0), NULL)));
		}
	}
	g_static_rec_mutex_unlock (&self->priv->__lock_channels);
	return val;
}


GFile* dvb_channel_list_get_channels_file (DVBChannelList* self) {
	g_return_val_if_fail (DVB_IS_CHANNEL_LIST (self), NULL);
	return self->priv->_channels_file;
}


static void dvb_channel_list_set_channels_file (DVBChannelList* self, GFile* value) {
	GFile* _tmp2;
	GFile* _tmp1;
	g_return_if_fail (DVB_IS_CHANNEL_LIST (self));
	_tmp2 = NULL;
	_tmp1 = NULL;
	self->priv->_channels_file = (_tmp2 = (_tmp1 = value, (_tmp1 == NULL ? NULL : g_object_ref (_tmp1))), (self->priv->_channels_file == NULL ? NULL : (self->priv->_channels_file = (g_object_unref (self->priv->_channels_file), NULL))), _tmp2);
	g_object_notify (((GObject *) (self)), "channels-file");
}


gint dvb_channel_list_get_size (DVBChannelList* self) {
	g_return_val_if_fail (DVB_IS_CHANNEL_LIST (self), 0);
	return gee_map_get_size (GEE_MAP (self->channels));
}


static GObject * dvb_channel_list_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	DVBChannelListClass * klass;
	GObjectClass * parent_class;
	DVBChannelList * self;
	klass = DVB_CHANNEL_LIST_CLASS (g_type_class_peek (DVB_TYPE_CHANNEL_LIST));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = DVB_CHANNEL_LIST (obj);
	{
		GeeHashMap* _tmp0;
		_tmp0 = NULL;
		self->channels = (_tmp0 = gee_hash_map_new (G_TYPE_UINT, NULL, NULL, DVB_TYPE_CHANNEL, ((GBoxedCopyFunc) (g_object_ref)), g_object_unref, g_direct_hash, g_direct_equal, g_direct_equal), (self->channels == NULL ? NULL : (self->channels = (g_object_unref (self->channels), NULL))), _tmp0);
	}
	return obj;
}


static void dvb_channel_list_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	DVBChannelList * self;
	self = DVB_CHANNEL_LIST (object);
	switch (property_id) {
		case DVB_CHANNEL_LIST_CHANNELS_FILE:
		g_value_set_object (value, dvb_channel_list_get_channels_file (self));
		break;
		case DVB_CHANNEL_LIST_SIZE:
		g_value_set_int (value, dvb_channel_list_get_size (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void dvb_channel_list_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	DVBChannelList * self;
	self = DVB_CHANNEL_LIST (object);
	switch (property_id) {
		case DVB_CHANNEL_LIST_CHANNELS_FILE:
		dvb_channel_list_set_channels_file (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void dvb_channel_list_class_init (DVBChannelListClass * klass) {
	dvb_channel_list_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (DVBChannelListPrivate));
	G_OBJECT_CLASS (klass)->get_property = dvb_channel_list_get_property;
	G_OBJECT_CLASS (klass)->set_property = dvb_channel_list_set_property;
	G_OBJECT_CLASS (klass)->constructor = dvb_channel_list_constructor;
	G_OBJECT_CLASS (klass)->finalize = dvb_channel_list_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_LIST_CHANNELS_FILE, g_param_spec_object ("channels-file", "channels-file", "channels-file", G_TYPE_FILE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), DVB_CHANNEL_LIST_SIZE, g_param_spec_int ("size", "size", "size", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void dvb_channel_list_gee_iterable_interface_init (GeeIterableIface * iface) {
	dvb_channel_list_gee_iterable_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_element_type = dvb_channel_list_real_get_element_type;
	iface->iterator = dvb_channel_list_real_iterator;
}


static void dvb_channel_list_dvb_id_bus_channel_list_interface_init (DVBIDBusChannelListIface * iface) {
	dvb_channel_list_dvb_id_bus_channel_list_parent_iface = g_type_interface_peek_parent (iface);
	iface->GetChannels = dvb_channel_list_real_GetChannels;
	iface->GetRadioChannels = dvb_channel_list_real_GetRadioChannels;
	iface->GetTVChannels = dvb_channel_list_real_GetTVChannels;
	iface->GetChannelName = dvb_channel_list_real_GetChannelName;
	iface->GetChannelNetwork = dvb_channel_list_real_GetChannelNetwork;
	iface->IsRadioChannel = dvb_channel_list_real_IsRadioChannel;
}


static void dvb_channel_list_instance_init (DVBChannelList * self) {
	self->priv = DVB_CHANNEL_LIST_GET_PRIVATE (self);
	g_static_rec_mutex_init (&self->priv->__lock_channels);
}


static void dvb_channel_list_finalize (GObject* obj) {
	DVBChannelList * self;
	self = DVB_CHANNEL_LIST (obj);
	(self->priv->_channels_file == NULL ? NULL : (self->priv->_channels_file = (g_object_unref (self->priv->_channels_file), NULL)));
	g_static_rec_mutex_free (&self->priv->__lock_channels);
	(self->channels == NULL ? NULL : (self->channels = (g_object_unref (self->channels), NULL)));
	(self->priv->channelsfile == NULL ? NULL : (self->priv->channelsfile = (g_object_unref (self->priv->channelsfile), NULL)));
	G_OBJECT_CLASS (dvb_channel_list_parent_class)->finalize (obj);
}


GType dvb_channel_list_get_type (void) {
	static GType dvb_channel_list_type_id = 0;
	if (dvb_channel_list_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (DVBChannelListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) dvb_channel_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (DVBChannelList), 0, (GInstanceInitFunc) dvb_channel_list_instance_init, NULL };
		static const GInterfaceInfo gee_iterable_info = { (GInterfaceInitFunc) dvb_channel_list_gee_iterable_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		static const GInterfaceInfo dvb_id_bus_channel_list_info = { (GInterfaceInitFunc) dvb_channel_list_dvb_id_bus_channel_list_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
		dvb_channel_list_type_id = g_type_register_static (G_TYPE_OBJECT, "DVBChannelList", &g_define_type_info, 0);
		g_type_add_interface_static (dvb_channel_list_type_id, GEE_TYPE_ITERABLE, &gee_iterable_info);
		g_type_add_interface_static (dvb_channel_list_type_id, DVB_TYPE_ID_BUS_CHANNEL_LIST, &dvb_id_bus_channel_list_info);
	}
	return dvb_channel_list_type_id;
}




